function [W,x_comp,y]=convolutionWavelet(u,v,x,k,plotFlag)

% Determine the type of signal (vector or image)
[M,N] = size(u);
s=size(x);
M=s(1);
N1=s(2);
W = 0;
x_comp =0;
y=0;
if (N1==1)
    y = 0;
    % Vectors base u and v of the wavelet are passed by argument
    W=zeros(N);
    
    for j=1:N/2
        W(j,:) = circshift(u,[1 2*(j-1)]);
        W(N/2+j,:) = circshift(v,[1 2*(j-1)]);
    end
    %W
    % Check that the matrix is unitary
    if (plotFlag == 1)
        figure;
        imshow(W*W'),title('Matrix tranformation W*W'' is unitary');
        
        % Plot the vectors u and v of the Haar'Wavelet basis
        figure;
        u_min=min(u);
        u_max=max(u);
        v_min=min(v);
        v_max=max(v);
        y_min=min(u_min,v_min);
        y_max=max(u_max,v_max);
        subplot(1,2,1);plot(fftshift(u)), title(['vector u of size ', num2str(N)]);
        ylim([y_min y_max]);
        xlim([0 N]);
        subplot(1,2,2);plot(fftshift(v)), title(['vector v of size ', num2str(N)]);
        ylim([y_min y_max]);
        xlim([0 N]);
    end
else
    
    N = N1;
    if (M==1) % The signal is a vector
        % Vectors base u and v of the wavelet        

        W=zeros(N);
        
        for j=1:N/2
            W(j,:) = circshift(u,[1 2*(j-1)]);
            W(N/2+j,:) = circshift(v,[1 2*(j-1)]);
        end
        
        Winv = W';
        
        % Daubechies' D6 wavelet transform of x
        y1=circshift(ifft(fft(fliplr(u)).*fft(x)), [1,1]);
        y2=circshift(ifft(fft(fliplr(v)).*fft(x)), [1,1]);
        
        y=[y1(1:2:N) y2(1:2:N)];
        
        yd=sort(abs(y),'descend');
        limit=fix(k*N/100);
        y_comp=y;
        y_comp(abs(y_comp)<yd(limit))=0;
        
        x_comp=Winv*y_comp';
        
        if (plotFlag == 1)
            figure;
            subplot(3,1,1);plot(x), title('Original signal x');
            subplot(3,1,2);plot(y), title('Haar transform');
            %ylim([min(x_comp) max(x_comp)]);
            subplot(3,1,3);plot(x_comp), title(['Inverse Haar transform with ' , num2str(k),'% of the largest coefficients']);
            ylim([min(x_comp) max(x_comp)]);
        end
    else
        % The signal is an image
        % 2_D Daubechies' D6 transform
        % Transform for rows

        % Daubechies' D6 wavelet transform of rows of x
        y1=zeros(M,N);
        y2=zeros(M,N);
        
        for k=1:M
            y1(k,:)=circshift(ifft((fft(fliplr(u)).*fft(x(k,:)))), [1,1]);
            y2(k,:)=circshift(ifft((fft(fliplr(v)).*fft(x(k,:)))), [1,1]);
        end
        
        y=[y1(:,1:2:N) y2(:,1:2:N)];

        % Transform for columns
        % Haar wavelet transform of columns of x
        y1=zeros(M,N);
        y2=zeros(M,N);
        
        for k=1:N
            y1(:,k)=circshift(ifft((fft(fliplr(u)).*fft(y(:,k)'))), [1,1]);
            y2(:,k)=circshift(ifft((fft(fliplr(v)).*fft(y(:,k)'))), [1,1]);
        end
        
        y=[y1(1:2:M,:);y2(1:2:M,:)];
                
        if (plotFlag == 1)
            figure;
            imshow(y),title('Haar transform');
        end
    end
end

% To check that is a wavelet and plot u,v
% [W,x_comp,y]=convolutionWavelet(u,v,x,k,plotFlag);

% To compute Haar transform in 1-dimension
% t=0:499;x=sin(2*pi/150*t)+sin(2*pi/20*t);[W,x_comp,y]=E18_bonet_1(u,v,x,256,40,1);

% To compute Haar transform in 2-dimensions
% x=imread('lenna.jpg'); x=im2double(x); x=rgb2gray(x);[W,x_comp,y]=E18_bonet_1(x,256,40,1);